==> ggcyto_1.12.0.tar.gz: ggcyto/R/ggcyto.R <==
#' Plot cytometry data using the ggcyto API
#'
#' \code{ggcyto()} initializes a ggcyto object that inherits ggplot class.
#' Similarly the + operator can be used to add layers to the
#' existing ggcyto object. 
#'
#' To invoke \code{ggcyto}:
#' \itemize{
#'    \item \code{ggcyto(fs, aes(x, y, <other aesthetics>))}
#'   }
#'
#'  @return ggcyto object      
#' @import methods ggplot2 flowCore ncdfFlow flowWorkspace
#' @export
#' @keywords internal
#' @param data The data source. A core cytometry data structure. (flowSet,flowFrame, GatingSet or GatingHierarchy)
#' @param ... other arguments passed to specific methods
#' @examples
#' 
#' data(GvHD)
#' fs <- GvHD[1:3]
#' #construct the `ggcyto` object (inherits from `ggplot` class)
#' p <- ggcyto(fs, aes(x = `FSC-H`)) 
#' p + geom_histogram() 
#'
#' # display density/area
#' p + geom_density()
#' p + geom_area(stat = "density") 
#' 
#' # 2d scatter plot
#' p <- ggcyto(fs, aes(x = `FSC-H`, y =  `SSC-H`))
#' p + geom_hex(bins = 128)
#' # do it programatically through aes_string and variables
#' col1 <- "`FSC-H`" #note that the dimension names with special characters needs to be quoted by backticks
#' col2 <- "`SSC-H`"
#' ggcyto(fs, aes_string(col1,col2)) + geom_hex()
ggcyto <- function(data = NULL, ...) UseMethod("ggcyto")


#' Reports whether x is a ggcyto object
#' @param x An object to test
#' @return TRUE/FALSE
#' @examples 
#' data(GvHD)
#' fs <- GvHD[1:2]
#' p <- ggcyto(fs, aes(x = `FSC-H`))
#' is.ggcyto(p)
#' @export
is.ggcyto <- function(x) inherits(x, "ggcyto")

#' @export
#' @rdname  ggcyto
ggcyto.default <- function(data = NULL, mapping = aes(), ...) {
  ggcyto.flowSet(fortify_fs(data, ...), mapping)
}

#' Draw ggcyto on current graphics device.
#'
#' A wrapper for print.ggplot. It converts the ggcyto to conventional ggplot object before printing it.
#' This is usually invoked automatically when a ggcyto object is returned to R console.
#' 
#' @return nothing
#' @param x ggcyto object to display
#' @param ... other arguments not used by this method
#' 
#' @export
#' @method print ggcyto
print.ggcyto <- function(x, ...) {
  
    
    x <- ggplot2:::plot_clone(x) #clone plot to avoid tampering original x due to ther referenceClass x$scales
    x <- as.ggplot(x) 
    ggplot2:::print.ggplot(x)
}

#' @rdname print.ggcyto
#' @method plot ggcyto
#' @export
plot.ggcyto <- print.ggcyto

#--------These S4 methods exsits for plotting ggcyto object automatically in R console---------------#
#' @export
#' @rdname print.ggcyto
setMethod("print", c("ggcyto"), print.ggcyto)


#' @param object ggcyto object
#' @rdname print.ggcyto
#' @method show ggcyto
#' @export
show.ggcyto <- function(object){print(object)}

#' @rdname print.ggcyto
#' @method show ggcyto
#' @export
setMethod("show", "ggcyto", show.ggcyto)

#' It fortifies the data, fills some default settings and returns a regular ggplot object.
#' 
#' The orginal data format is preserved during the ggcyo constructor because they still need to be used during the plot building process.
#' This function is usually called automatically in the print/plot method of ggycyto. Sometime it is useful to coerce it to ggplot explictily 
#' by user so that it can be used as a regular ggplot object.
#' 
#' @param x ggcyto object with the data that has not yet been fortified to data.frame.
#' 
#' @return ggplot object
#' @examples 
#' data(GvHD)
#' fs <- GvHD[1:3]
#' #construct the `ggcyto` object (inherits from `ggplot` class)
#' p <- ggcyto(fs, aes(x = `FSC-H`)) + geom_histogram() 
#' class(p) # a ggcyto object
#' p$data # data has not been fortified
#' p1 <- as.ggplot(p) # convert it to a ggplot object explictily 
#' class(p1) 
#' p1$data # data is fortified
#' @export
as.ggplot <- function(x){

  #####################
  #lazy-fortifying the plot data
  #####################
  dims <- attr(x[["data"]], "dims")
  aes_names <- dims[, axis]
  chnls <- dims[, name]
  
  instrument_range <- x[["instrument_range"]]
  dtype <- class(x[["data"]])
  gs <- fs <- NULL
  #data needs to be fortified here if geom_gate was not added
  if(dtype != "data.table"){
    if(dtype %in% c("GatingSet", "GatingSetList")){#check if it is currently gs
      gs <- x[["data"]]
      fs <- fortify_fs(gs)
      
    }else
      fs <- x[["data"]]
    x[["data"]] <- fortify(fs)
    data_range <- apply(x[["data"]][, chnls, with = FALSE], 2, range)
    rownames(data_range) <- c("min", "max")  
  }else
    data_range <- x[["data_range"]]
  
  #post process geom_hex layers 
  for(i in seq_along(x$layers))
  {
    e2 <- x$layers[[i]]
    #with the one that is based on data limits to avoid oversized bins caused by exagerated gates
    if(is(e2$geom, "GeomHex"))
    {
      
      if(is.null(e2$stat_params[["binwidth"]]))
      {
        dummy_scales <- sapply(c("x", "y"), function(i)scale_x_continuous(limits = as.vector(data_range[,dims[axis==i, name]])))
        e2$stat_params[["binwidth"]] <- ggplot2:::hex_binwidth(e2$stat_params[["bins"]], dummy_scales)
        x$layers[[i]] <- e2
      }
    }
  }
  #####################
  #update default scales
  #####################
  breaks <- x[["axis_inverse_trans"]]
  stats_limits <- list()
  trans <- list()
  for(this_aes in aes_names)
  {
    dim <- dims[axis == this_aes, name]
    # set limits
    if(!x$scales$has_scale(this_aes))
    {
      #add new one if not present 
      new.scale <- ggplot2:::make_scale("continuous", this_aes)
      
      x <- ggplot2:::`+.gg`(x, new.scale)      
      
    }
    ind <- which(x$scales$find(this_aes))
    #apply lazy limits setting
    par_limits <- x$ggcyto_pars[["limits"]]
    if(is.character(par_limits)&&par_limits == "data")
    {
      this_limits <- data_range[, dim]
      x$coordinates[["limits"]][[this_aes]] <- this_limits
      
    }else if(!is.null(par_limits))
      stop("How did you end up here?")
        
    stats_limits[[dim]] <- x$coordinates[["limits"]][[this_aes]]
    #update breaks and labels
    thisBreaks <- breaks[[this_aes]]
    if(!is.null(thisBreaks)){
      # set limits
      if(!x$scales$has_scale(this_aes))
      {
        #add new one if not present 
        new.scale <- ggplot2:::make_scale("continuous", this_aes)
        x <- x + new.scale
      }
      ind <- which(x$scales$find(this_aes))
      x$scales$scales[[ind]]$breaks <- thisBreaks[["at"]]
      x$scales$scales[[ind]]$labels <- thisBreaks[["label"]]  
    }
    
  }
  if(!is.null(data_range)&&length(stats_limits)!=0)
  {
    stats_limits <- as.data.frame(stats_limits, check.names = FALSE)
    stats_limits[["density"]] <- c(0,1e-4)
    
  }else
    stats_limits <- NULL
  
  #retrospect geom_hex layer to fix binwidth
  for(i in seq_along(x$layers))
  {
    e2 <- x$layers[[i]]
    #override default bindwidth that is based on the entire scale limits
    #with the one that is based on data limits to avoid oversized bins caused by exagerated gates
    if(is(e2$geom, "GeomHex"))
    {
      bw <- e2$stat_params[["binwidth"]]
      bins <- e2$stat_params[["bins"]]
      if(is.null(bins)||length(bins)==0)
      {
        bins <- formals(stat_bin_hex)[["bins"]]
      }
      
      if(is.null(bw)||length(bw)==0)
      {
        dummy_scales <- sapply(c("x", "y"), function(i)scale_x_continuous(limits = as.vector(data_range[,dims[axis==i, name]])))
        e2$stat_params[["binwidth"]] <- ggplot2:::hex_binwidth(bins, dummy_scales)
        x$layers[[i]] <- e2  
      }
      
    }
    
  }
  #lazy parsing stats layer since the stats_limits is set at the end
  for(e2 in x[["GeomStats"]])
  {
    gate <- e2[["gate"]]
    #parse the gate from the each gate layer if it is not present in the current geom_stats layer
    if(is.null(gate))
    {
      
      pd <- .pd2dt(pData(fs))
      gates_parsed <- lapply(x$layers, function(layer){
        
        if(is.geom_gate_filterList(layer))#restore filter from fortified data.frame
          .dataframe2filterList(layer$data, colnames(pd))
        else
          NULL
      })
      #remove NULL elements
      gates_parsed <- flowWorkspace:::compact(gates_parsed)
    }else{
      gates_parsed <- list(gate)
    }             
    
    
    if(length(gates_parsed) == 0)
      stop("geom_gate layer must be added before geom_stats!")
    
    
    # compute pop stats for each gate layer and 
    value <- e2[["value"]]
    stat_type <- e2[["type"]]
    

    #add default density range
    #In order to ensure the stats visiblity
    #try to put it closer to zero because we don't know the actual density range
    if(!is.null(data_range))
      data_range <- as.data.frame(data_range)
    
    
    negated <- e2[["negated"]]
    adjust <- e2[["adjust"]]
    digits <- e2[["digits"]]
    if(length(trans)>0)
    {
      translist <- lapply(trans, function(t)t[["transform"]])
      translist <- transformList(names(translist), translist)
      inverselist <- lapply(trans, function(t)t[["inverse"]])
    }
    if(length(trans)>0&&is.null(value))#means fs will be used to compute stats and thus needs to be scaled properly
    {
      suppressMessages(fs <- transform(fs, translist))
    }
    for(gate in gates_parsed){
      if(length(trans)>0)
        gate <- transform(gate, translist)
      stats <- compute_stats(fs, gate, type = stat_type, value = value, data_range = data_range, limits = stats_limits, negated = negated, adjust = adjust, digits = digits)
      
      #restore the stats dimensions to raw scale
      if(length(trans)>0)
      {
        for(param in names(inverselist))
        {
          thisTrans <- inverselist[[param]]
          v <- thisTrans(stats[[param]])
          stats[, (param) := v]
        }  
      }
      
        
      # instantiate the new stats layer
      thisCall <- quote(geom_label(data = stats))
      # copy all the other parameters
      thisCall <-  as.call(c(as.list(thisCall), e2[["geom_label_params"]]))
      
      e2.new <- eval(thisCall)
      attr(e2.new, "is.recorded") <- TRUE
      # update aes
      stats_mapping <- aes_string(label = "value")
      #add y aes for 1d density plot
      dims <- x$mapping[grepl("[x|y]", names(x$mapping))]
      dims <- sapply(dims, quo_name)
      if(length(dims) == 1)
        stats_mapping <- defaults(stats_mapping, aes(y = density))
      e2.new$mapping <- defaults(e2.new$mapping, stats_mapping)  
      
      x <- ggplot2:::`+.gg`(x, e2.new)      
    }
  }
  
  #strip the ggcyto class attributes
  asS3(x)
}

==> ggcyto_1.14.1.tar.gz: ggcyto/R/ggcyto.R <==
#' Plot cytometry data using the ggcyto API
#'
#' \code{ggcyto()} initializes a ggcyto object that inherits ggplot class.
#' Similarly the + operator can be used to add layers to the
#' existing ggcyto object. 
#'
#' To invoke \code{ggcyto}:
#' \itemize{
#'    \item \code{ggcyto(fs, aes(x, y, <other aesthetics>))}
#'   }
#'
#'  @return ggcyto object      
#' @import methods ggplot2 flowCore ncdfFlow flowWorkspace
#' @export
#' @keywords internal
#' @param data The data source. A core cytometry data structure. (flowSet,flowFrame, GatingSet or GatingHierarchy)
#' @param ... other arguments passed to specific methods
#' @examples
#' 
#' data(GvHD)
#' fs <- GvHD[1:3]
#' #construct the `ggcyto` object (inherits from `ggplot` class)
#' p <- ggcyto(fs, aes(x = `FSC-H`)) 
#' p + geom_histogram() 
#'
#' # display density/area
#' p + geom_density()
#' p + geom_area(stat = "density") 
#' 
#' # 2d scatter plot
#' p <- ggcyto(fs, aes(x = `FSC-H`, y =  `SSC-H`))
#' p + geom_hex(bins = 128)
#' # do it programatically through aes_string and variables
#' col1 <- "`FSC-H`" #note that the dimension names with special characters needs to be quoted by backticks
#' col2 <- "`SSC-H`"
#' ggcyto(fs, aes_string(col1,col2)) + geom_hex()
ggcyto <- function(data = NULL, ...) UseMethod("ggcyto")


#' Reports whether x is a ggcyto object
#' @param x An object to test
#' @return TRUE/FALSE
#' @examples 
#' data(GvHD)
#' fs <- GvHD[1:2]
#' p <- ggcyto(fs, aes(x = `FSC-H`))
#' is.ggcyto(p)
#' @export
is.ggcyto <- function(x) inherits(x, "ggcyto")

#' @export
#' @rdname  ggcyto
ggcyto.default <- function(data = NULL, mapping = aes(), ...) {
  ggcyto.flowSet(fortify_fs(data, ...), mapping)
}

#' Draw ggcyto on current graphics device.
#'
#' A wrapper for print.ggplot. It converts the ggcyto to conventional ggplot object before printing it.
#' This is usually invoked automatically when a ggcyto object is returned to R console.
#' 
#' @return nothing
#' @param x ggcyto object to display
#' @param ... other arguments not used by this method
#' 
#' @export
#' @method print ggcyto
print.ggcyto <- function(x, ...) {
  
    
    x <- ggplot2:::plot_clone(x) #clone plot to avoid tampering original x due to ther referenceClass x$scales
    x <- as.ggplot(x) 
    ggplot2:::print.ggplot(x)
}

#' @rdname print.ggcyto
#' @method plot ggcyto
#' @export
plot.ggcyto <- print.ggcyto

#--------These S4 methods exsits for plotting ggcyto object automatically in R console---------------#
#' @export
#' @rdname print.ggcyto
setMethod("print", c("ggcyto"), print.ggcyto)


#' @param object ggcyto object
#' @rdname print.ggcyto
#' @method show ggcyto
#' @export
show.ggcyto <- function(object){print(object)}

#' @rdname print.ggcyto
#' @method show ggcyto
#' @export
setMethod("show", "ggcyto", show.ggcyto)

#' It fortifies the data, fills some default settings and returns a regular ggplot object.
#' 
#' The orginal data format is preserved during the ggcyo constructor because they still need to be used during the plot building process.
#' This function is usually called automatically in the print/plot method of ggycyto. Sometime it is useful to coerce it to ggplot explictily 
#' by user so that it can be used as a regular ggplot object.
#' 
#' @param x ggcyto object with the data that has not yet been fortified to data.frame.
#' 
#' @return ggplot object
#' @examples 
#' data(GvHD)
#' fs <- GvHD[1:3]
#' #construct the `ggcyto` object (inherits from `ggplot` class)
#' p <- ggcyto(fs, aes(x = `FSC-H`)) + geom_histogram() 
#' class(p) # a ggcyto object
#' p$data # data has not been fortified
#' p1 <- as.ggplot(p) # convert it to a ggplot object explictily 
#' class(p1) 
#' p1$data # data is fortified
#' @export
as.ggplot <- function(x){

  #####################
  #lazy-fortifying the plot data
  #####################
  dims <- attr(x[["data"]], "dims")
  aes_names <- dims[, axis]
  chnls <- dims[, name]
  
  instrument_range <- x[["instrument_range"]]
  dtype <- class(x[["data"]])
  gs <- fs <- NULL
  #data needs to be fortified here if geom_gate was not added
  if(dtype != "data.table"){
    if(dtype %in% c("GatingSet", "GatingSetList")){#check if it is currently gs
      gs <- x[["data"]]
      fs <- fortify_fs(gs)
      
    }else
      fs <- x[["data"]]
    x[["data"]] <- fortify(fs)
    data_range <- apply(x[["data"]][, chnls, with = FALSE], 2, range)
    rownames(data_range) <- c("min", "max")  
  }else
    data_range <- x[["data_range"]]
  
  #post process geom_hex layers 
  for(i in seq_along(x$layers))
  {
    e2 <- x$layers[[i]]
    #with the one that is based on data limits to avoid oversized bins caused by exagerated gates
    if(is(e2$geom, "GeomHex"))
    {
      
      if(is.null(e2$stat_params[["binwidth"]]))
      {
        transformed_range <- data_range
        for(col in c("x","y")){
          if(!is.null(x$scales$get_scales(col)$secondary.axis)){
            transformed_range[, dims[axis==col, name]] <- x$scales$get_scales(col)$transform(transformed_range[,dims[axis==col, name]]) 
          }
        }
        dummy_scales <- sapply(c("x", "y"), function(i) scale_x_continuous(limits = as.vector(transformed_range[,dims[axis==i, name]])))
        e2$stat_params[["binwidth"]] <- ggplot2:::hex_binwidth(e2$stat_params[["bins"]], dummy_scales)
        x$layers[[i]] <- e2
      }
    }
  }
  #####################
  #update default scales
  #####################
  breaks <- x[["axis_inverse_trans"]]
  stats_limits <- list()
  trans <- list()
  for(this_aes in aes_names)
  {
    dim <- dims[axis == this_aes, name]
    # set limits
    if(!x$scales$has_scale(this_aes))
    {
      #add new one if not present 
      new.scale <- ggplot2:::make_scale("continuous", this_aes)
      
      x <- ggplot2:::`+.gg`(x, new.scale)      
      
    }
    ind <- which(x$scales$find(this_aes))
    #apply lazy limits setting
    par_limits <- x$ggcyto_pars[["limits"]]
    if(is.character(par_limits)&&par_limits == "data")
    {
      this_limits <- data_range[, dim]
      x$coordinates[["limits"]][[this_aes]] <- this_limits
      
    }else if(!is.null(par_limits))
      stop("How did you end up here?")
        
    stats_limits[[dim]] <- x$coordinates[["limits"]][[this_aes]]
    #update breaks and labels
    thisBreaks <- breaks[[this_aes]]
    if(!is.null(thisBreaks)){
      # set limits
      if(!x$scales$has_scale(this_aes))
      {
        #add new one if not present 
        new.scale <- ggplot2:::make_scale("continuous", this_aes)
        x <- x + new.scale
      }
      ind <- which(x$scales$find(this_aes))
      x$scales$scales[[ind]]$breaks <- thisBreaks[["at"]]
      x$scales$scales[[ind]]$labels <- thisBreaks[["label"]]  
    }
    
  }
  if(!is.null(data_range)&&length(stats_limits)!=0)
  {
    stats_limits <- as.data.frame(stats_limits, check.names = FALSE)
    stats_limits[["density"]] <- c(0,1e-4)
    
  }else
    stats_limits <- NULL
  
  #retrospect geom_hex layer to fix binwidth
  for(i in seq_along(x$layers))
  {
    e2 <- x$layers[[i]]
    #override default bindwidth that is based on the entire scale limits
    #with the one that is based on data limits to avoid oversized bins caused by exagerated gates
    if(is(e2$geom, "GeomHex"))
    {
      bw <- e2$stat_params[["binwidth"]]
      bins <- e2$stat_params[["bins"]]
      if(is.null(bins)||length(bins)==0)
      {
        bins <- formals(stat_bin_hex)[["bins"]]
      }
      
      if(is.null(bw)||length(bw)==0)
      {
        transformed_range <- data_range
        for(col in c("x","y")){
          if(!is.null(x$scales$get_scales(col)$secondary.axis)){
            transformed_range[, dims[axis==col, name]] <- x$scales$get_scales(col)$transform(transformed_range[,dims[axis==col, name]]) 
          }
        }
        dummy_scales <- sapply(c("x", "y"), function(i)scale_x_continuous(limits = as.vector(transformed_range[,dims[axis==i, name]])))
        e2$stat_params[["binwidth"]] <- ggplot2:::hex_binwidth(bins, dummy_scales)
        x$layers[[i]] <- e2
      }
      
    }
    
  }
  #lazy parsing stats layer since the stats_limits is set at the end
  for(e2 in x[["GeomStats"]])
  {
    gate <- e2[["gate"]]
    #parse the gate from the each gate layer if it is not present in the current geom_stats layer
    if(is.null(gate))
    {
      
      pd <- .pd2dt(pData(fs))
      gates_parsed <- lapply(x$layers, function(layer){
        
        if(is.geom_gate_filterList(layer))#restore filter from fortified data.frame
          .dataframe2filterList(layer$data, colnames(pd))
        else
          NULL
      })
      #remove NULL elements
      gates_parsed <- flowWorkspace:::compact(gates_parsed)
    }else{
      gates_parsed <- list(gate)
    }             
    
    
    if(length(gates_parsed) == 0)
      stop("geom_gate layer must be added before geom_stats!")
    
    
    # compute pop stats for each gate layer and 
    value <- e2[["value"]]
    stat_type <- e2[["type"]]
    

    #add default density range
    #In order to ensure the stats visiblity
    #try to put it closer to zero because we don't know the actual density range
    if(!is.null(data_range))
      data_range <- as.data.frame(data_range)
    
    
    negated <- e2[["negated"]]
    adjust <- e2[["adjust"]]
    digits <- e2[["digits"]]
    if(length(trans)>0)
    {
      translist <- lapply(trans, function(t)t[["transform"]])
      translist <- transformList(names(translist), translist)
      inverselist <- lapply(trans, function(t)t[["inverse"]])
    }
    if(length(trans)>0&&is.null(value))#means fs will be used to compute stats and thus needs to be scaled properly
    {
      suppressMessages(fs <- transform(fs, translist))
    }
    for(gate in gates_parsed){
      if(length(trans)>0)
        gate <- transform(gate, translist)
      stats <- compute_stats(fs, gate, type = stat_type, value = value, data_range = data_range, limits = stats_limits, negated = negated, adjust = adjust, digits = digits)
      
      #restore the stats dimensions to raw scale
      if(length(trans)>0)
      {
        for(param in names(inverselist))
        {
          thisTrans <- inverselist[[param]]
          v <- thisTrans(stats[[param]])
          stats[, (param) := v]
        }  
      }
      
        
      # instantiate the new stats layer
      thisCall <- quote(geom_label(data = stats))
      # copy all the other parameters
      thisCall <-  as.call(c(as.list(thisCall), e2[["geom_label_params"]]))
      
      e2.new <- eval(thisCall)
      attr(e2.new, "is.recorded") <- TRUE
      # update aes
      stats_mapping <- aes_string(label = "value")
      #add y aes for 1d density plot
      dims <- x$mapping[grepl("[x|y]", names(x$mapping))]
      dims <- sapply(dims, quo_name)
      if(length(dims) == 1)
        stats_mapping <- defaults(stats_mapping, aes(y = density))
      e2.new$mapping <- defaults(e2.new$mapping, stats_mapping)  
      
      x <- ggplot2:::`+.gg`(x, e2.new)      
    }
  }
  
  #strip the ggcyto class attributes
  asS3(x)
}
